import json


# get_markers requests all marker data or valid markers, converts the data to json, and writes
# the data to the response object
def get_all_markers(self, cursor, valid: bool):
    try:
        # NB: interval temporarily hard coded to 5 days
        if valid:
            cursor.execute('''
                SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, d.Latitude, d.Longitude,
                       d.IceTop, d.IceBottom, d.CalculatedThickness, d.Accuracy, s.SensorType, s.Active,
                       c.CornerID, c.CornerLatitude, c.CornerLongitude, b.Name
                FROM Measurement m
                INNER JOIN Sensor s ON m.SensorID = s.SensorID
                INNER JOIN Data d ON m.MeasurementID = d.MeasurementID
                LEFT JOIN Corner c ON m.MeasurementID = c.MeasurementID
                INNER JOIN BodyOfWater b ON m.WaterBodyName = b.Name
                WHERE m.TimeMeasured > DATE_SUB(NOW(), INTERVAL 5 DAY);
            ''')
        else:
            cursor.execute('''
                SELECT m.MeasurementID, m.SensorID, m.TimeMeasured, d.Latitude, d.Longitude,
                       d.IceTop, d.IceBottom, d.CalculatedThickness, d.Accuracy, s.SensorType, s.Active,
                       c.CornerID, c.CornerLatitude, c.CornerLongitude, b.Name
                FROM Measurement m
                INNER JOIN Sensor s ON m.SensorID = s.SensorID
                INNER JOIN Data d ON m.MeasurementID = d.MeasurementID
                LEFT JOIN Corner c ON m.MeasurementID = c.MeasurementID
                INNER JOIN BodyOfWater b ON m.WaterBodyName = b.Name
            ''')

        rows = cursor.fetchall()

        measurement_data = {}

        # Iterate over the fetched rows
        for row in rows:
            measurement_id = row[0]

            # Create a data object for current row
            data_object = {
                'Latitude': row[3],
                'Longitude': row[4],
                'IceTop': row[5],
                'IceBottom': row[6],
                'CalculatedThickness': row[7],
                'Accuracy': row[8]
            }

            # Create a corner object for current row
            corner_object = {
                'Latitude': row[12],
                'Longitude': row[13]
            }

            # Check if measurement ID already exists in measurement_data
            if measurement_id in measurement_data:
                # Check if the data object already exists in the list
                if data_object not in measurement_data[measurement_id]['Data']:
                    measurement_data[measurement_id]['Data'].append(data_object)

                # Check if the corner object already exists in the list
                if corner_object not in measurement_data[measurement_id]['Corners']:
                    measurement_data[measurement_id]['Corners'].append(corner_object)
            else:
                # Create a new entry for measurement_id if it does not already exist in the list
                measurement_data[measurement_id] = {
                    'MeasurementID': measurement_id,
                    'TimeMeasured': row[2],
                    'Sensor': {
                        'SensorID': row[1],
                        'SensorType': row[9],
                        'Active': bool(row[10])
                    },
                    'Data': [data_object],
                    'Corners': [corner_object]
                }

        # Convert dictionary values to list of measurements
        data = list(measurement_data.values())

        if len(rows) == 0 or len(data) == 0:  # Return 500 and empty list if no data is found
            print(f"An error occurred while querying the database")
            resp_code = 500
            marker_data = '[]'
        else:
            resp_code = 200
            # Convert list of dictionaries to JSON
            marker_data = json.dumps(data, indent=4)

    except Exception as e:
        print(f"An error occurred while querying the database: {e}")
        resp_code = 500
        marker_data = '[]'

    # Set headers
    self.send_response(resp_code)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write marker data to response object
    self.wfile.write(marker_data.encode('utf-8'))

